@@ -27,7 +27,7 @@ class node():
     def assign_optim(self, optim):
         self.optim = optim
     
-    def local_update(self, local_steps, device):
+    def local_update_steps(self, local_steps, device):
         # print(len(self.train), self.step)
         if len(self.train) - self.step > local_steps:
             for k, (inputs, labels) in enumerate(self.train):
@@ -63,29 +63,10 @@ class node():
         self.loss.backward()
         self.optim.step()
         # self.train_steps+=1
-    # def local_update(self, local_epochs, device):
-    #     # local_steps may be better!!
-    #     running_loss = 0
-    #     for j in range(local_epochs):
-    #         for k, (inputs, labels) in enumerate(self.train):
-    #             inputs = inputs.to(device)
-    #             labels = labels.to(device)
-    #             # zero the parameter gradients
-    #             self.model.zero_grad()
-    #             # forward + backward + optimize
-    #             outputs = self.model(inputs)
-    #             # optim
-    #             self.loss = self.objective(outputs, labels)
-    #             self.loss.backward()
-    #             self.optim.step()
-    #             # print
-    #             running_loss += self.loss.item()
-    #             if (k+1) % 100 == 0:    # print every 100 mini-batches
-    #                 print('[%d %d] node_%d loss: %.3f' %
-    #                       (j, k+1, self.id, running_loss/20))
-    #                 running_loss = 0
 
-    def ditto_local_update(self, local_epochs, device, server, lam):
+    def local_update_epochs(self, local_epochs, device):
+        # local_steps may be better!!
+        running_loss = 0
         for j in range(local_epochs):
             for k, (inputs, labels) in enumerate(self.train):
                 inputs = inputs.to(device)
@@ -95,12 +76,52 @@ class node():
                 # forward + backward + optimize
                 outputs = self.model(inputs)
                 # optim
-                reg = 0
-                for p,q in zip(self.model.parameters(),server.model.parameters()):
-                    reg += torch.norm((p-q),2)
-                self.loss = self.objective(outputs, labels) + lam*reg/2
+                self.loss = self.objective(outputs, labels)
                 self.loss.backward()
                 self.optim.step()
+                # print
+                running_loss += self.loss.item()
+                if (k+1) % 100 == 0:    # print every 100 mini-batches
+                    print('[%d %d] node_%d loss: %.3f' %
+                          (j, k+1, self.id, running_loss/20))
+                    running_loss = 0
+
+    def local_update_ditto(self, local_steps, device, server, lam):
+        # print(len(self.train), self.step)
+        if len(self.train) - self.step > local_steps:
+            for k, (inputs, labels) in enumerate(self.train):
+                if k < self.step or k >= self.step + local_steps:
+                    continue
+                self.train_single_step_ditto(inputs, labels, device, server, lam)
+            self.step = self.step + local_steps
+        else:
+            for k, (inputs, labels) in enumerate(self.train):
+                if k < self.step:
+                    continue
+                self.train_single_step_ditto(inputs, labels, device, server, lam)
+            for j in range((local_steps-len(self.train)+self.step)//len(self.train)):
+                for k, (inputs, labels) in enumerate(self.train):
+                    self.train_single_step_ditto(inputs, labels, device, server, lam)
+            for k, (inputs, labels) in enumerate(self.train):
+                if k >=(local_steps-len(self.train)+self.step)%len(self.train):
+                    continue
+                self.train_single_step_ditto(inputs, labels, device, server, lam)
+            self.step = (local_steps-len(self.train)+self.step)%len(self.train)
+
+    def train_single_step_ditto(self, inputs, labels, device, server, lam):
+        inputs = inputs.to(device)
+        labels = labels.to(device)
+        # zero the parameter gradients
+        self.model.zero_grad()
+        # forward + backward + optimize
+        outputs = self.model(inputs)
+        # optim
+        reg = 0
+        for p,q in zip(self.model.parameters(),server.model.parameters()):
+            reg += torch.norm((p-q),2)
+        self.loss = self.objective(outputs, labels) + lam*reg/2
+        self.loss.backward()
+        self.optim.step()
 
     def local_test(self, device):
         correct = 0
